/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * 你可以在不违反许可协议的情况下使用此文件。
 * 你可以在以下地址获取许可证的副本：
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * 除非适用法律要求或书面同意，软件
 * 根据许可证分发是按“原样”基础进行的，
 * 不附带任何明示或暗示的担保或条件。
 * 请参阅许可证以获取特定语言的权限和
 * 限制。
 */

package net.micode.notes.data;

import android.net.Uri;

// 定义了一个名为Notes的类，包含了与笔记应用相关的常量和接口
public class Notes {
    // 定义了Content Provider的权限字符串
    public static final String AUTHORITY = "micode_notes";
    // 定义了日志标签字符串
    public static final String TAG = "Notes";

    // 定义了笔记类型的常量
    public static final int TYPE_NOTE = 0;  // 笔记类型
    public static final int TYPE_FOLDER = 1;  // 文件夹类型
    public static final int TYPE_SYSTEM = 2;  // 系统类型

    /**
     * 以下ID是系统文件夹的标识符
     * {@link Notes#ID_ROOT_FOLDER } 是默认文件夹
     * {@link Notes#ID_TEMPARAY_FOLDER } 用于不属于任何文件夹的笔记
     * {@link Notes#ID_CALL_RECORD_FOLDER} 用于存储通话记录
     * {@link Notes#ID_TRASH_FOLER} 用于存储已删除的笔记
     */
    public static final int ID_ROOT_FOLDER = 0;
    public static final int ID_TEMPARAY_FOLDER = -1;
    public static final int ID_CALL_RECORD_FOLDER = -2;
    public static final int ID_TRASH_FOLER = -3;

    // 定义了一些Intent的extra键名
    public static final String INTENT_EXTRA_ALERT_DATE = "net.micode.notes.alert_date";  // 提醒日期
    public static final String INTENT_EXTRA_BACKGROUND_ID = "net.micode.notes.background_color_id";  // 背景颜色ID
    public static final String INTENT_EXTRA_WIDGET_ID = "net.micode.notes.widget_id";  // 小部件ID
    public static final String INTENT_EXTRA_WIDGET_TYPE = "net.micode.notes.widget_type";  // 小部件类型
    public static final String INTENT_EXTRA_FOLDER_ID = "net.micode.notes.folder_id";  // 文件夹ID
    public static final String INTENT_EXTRA_CALL_DATE = "net.micode.notes.call_date";  // 通话日期

    // 定义了小部件类型的常量
    public static final int TYPE_WIDGET_INVALIDE = -1;  // 无效的小部件
    public static final int TYPE_WIDGET_2X = 0;  // 2x尺寸的小部件
    public static final int TYPE_WIDGET_4X = 1;  // 4x尺寸的小部件

    // 定义一个内部类DataConstants，包含一些与数据类型相关的常量
    public static class DataConstants {
        // 定义了文本笔记的数据类型
        public static final String NOTE = TextNote.CONTENT_ITEM_TYPE;
        // 定义了通话笔记的数据类型
        public static final String CALL_NOTE = CallNote.CONTENT_ITEM_TYPE;
    }

    /**
     * Uri用于查询所有的笔记和文件夹
     * 解析为：content://micode_notes/note
     */
    public static final Uri CONTENT_NOTE_URI = Uri.parse("content://" + AUTHORITY + "/note");

    /**
     * Uri用于查询数据
     * 解析为：content://micode_notes/data
     */
    public static final Uri CONTENT_DATA_URI = Uri.parse("content://" + AUTHORITY + "/data");

    // 定义了一个接口NoteColumns，包含了一些与笔记和文件夹相关的列名
    public interface NoteColumns {
        /**
         * 笔记或文件夹的唯一ID
         * <P> 类型：INTEGER (long) </P>
         */
        public static final String ID = "_id";

        /**
         * 笔记或文件夹的父ID
         * <P> 类型：INTEGER (long) </P>
         */
        public static final String PARENT_ID = "parent_id";

        /**
         * 笔记或文件夹的创建日期
         * <P> 类型：INTEGER (long) </P>
         */
        public static final String CREATED_DATE = "created_date";

        /**
         * 笔记或文件夹的最新修改日期
         * <P> 类型：INTEGER (long) </P>
         */
        public static final String MODIFIED_DATE = "modified_date";

        /**
         * 笔记的提醒日期
         * <P> 类型：INTEGER (long) </P>
         */
        public static final String ALERTED_DATE = "alert_date";

        /**
         * 文件夹的名称或笔记的文本内容
         * <P> 类型：TEXT </P>
         */
        public static final String SNIPPET = "snippet";

        /**
         * 笔记的小部件ID
         * <P> 类型：INTEGER (long) </P>
         */
        public static final String WIDGET_ID = "widget_id";

        /**
         * 笔记的小部件类型
         * <P> 类型：INTEGER (long) </P>
         */
        public static final String WIDGET_TYPE = "widget_type";

        /**
         * 笔记的背景颜色ID
         * <P> 类型：INTEGER (long) </P>
         */
        public static final String BG_COLOR_ID = "bg_color_id";

        /**
         * 对于文本笔记，它没有附件；对于多媒体笔记，至少有一个附件
         * <P> 类型：INTEGER </P>
         */
        public static final String HAS_ATTACHMENT = "has_attachment";

        /**
         * 文件夹中的笔记数量
         * <P> 类型：INTEGER (long) </P>
         */
        public static final String NOTES_COUNT = "notes_count";

        /**
         * 笔记或文件夹的类型
         * <P> 类型：INTEGER </P>
         */
        public static final String TYPE = "type";

        /**
         * 最后同步ID
         * <P> 类型：INTEGER (long) </P>
         */
        public static final String SYNC_ID = "sync_id";

        /**
         * 标志以指示是否本地修改过
         * <P> 类型：INTEGER </P>
         */
        public static final String LOCAL_MODIFIED = "local_modified";

        /**
         * 移动到临时文件夹之前的原始父ID
         * <P> 类型：INTEGER </P>
         */
        public static final String ORIGIN_PARENT_ID = "origin_parent_id";

        /**
         * Google任务ID
         * <P> 类型：TEXT </P>
         */
        public static final String GTASK_ID = "gtask_id";

        /**
         * 版本号
         * <P> 类型：INTEGER (long) </P>
         */
        public static final String VERSION = "version";
    }

    // 定义了一个接口DataColumns，包含了一些与数据相关的列名
    public interface DataColumns {
        /**
         * 数据的唯一ID
         * <P> 类型：INTEGER (long) </P>
         */
        public static final String ID = "_id";

        /**
         * 此行所代表项目的数据类型（MIME类型）。
         * <P> 类型：TEXT </P>
         */
        public static final String MIME_TYPE = "mime_type";

        /**
         * 引用此数据属于的笔记ID
         * <P> 类型：INTEGER (long) </P>
         */
        public static final String NOTE_ID = "note_id";

        /**
         * 数据的创建日期
         * <P> 类型：INTEGER (long) </P>
         */
        public static final String CREATED_DATE = "created_date";

        /**
         * 数据的最新修改日期
         * <P> 类型：INTEGER (long) </P>
         */
        public static final String MODIFIED_DATE = "modified_date";

        /**
         * 数据的内容
         * <P> 类型：TEXT </P>
         */
        public static final String CONTENT = "content";

        /**
         * 通用数据列，含义取决于 {@link #MIME_TYPE}，用于整型数据类型
         * <P> 类型：INTEGER </P>
         */
        public static final String DATA1 = "data1";

        /**
         * 通用数据列，含义取决于 {@link #MIME_TYPE}，用于整型数据类型
         * <P> 类型：INTEGER </P>
         */
        public static final String DATA2 = "data2";

        /**
         * 通用数据列，含义取决于 {@link #MIME_TYPE}，用于文本数据类型
         * <P> 类型：TEXT </P>
         */
        public static final String DATA3 = "data3";

        /**
         * 通用数据列，含义取决于 {@link #MIME_TYPE}，用于文本数据类型
         * <P> 类型：TEXT </P>
         */
        public static final String DATA4 = "data4";

        /**
         * 通用数据列，含义取决于 {@link #MIME_TYPE}，用于文本数据类型
         * <P> 类型：TEXT </P>
         */
        public static final String DATA5 = "data5";
    }

    // 定义了一个内部类TextNote，继承自DataColumns，表示文本笔记的数据结构
    public static final class TextNote implements DataColumns {
        /**
         * 模式以指示文本是否处于检查列表模式
         * <P> 类型：Integer 1:检查列表模式 0:正常模式 </P>
         */
        public static final String MODE = DATA1;

        // 定义了文本笔记模式的常量
        public static final int MODE_CHECK_LIST = 1;

        // 定义了文本笔记的MIME类型
        public static final String CONTENT_TYPE = "vnd.android.cursor.dir/text_note";
        public static final String CONTENT_ITEM_TYPE = "vnd.android.cursor.item/text_note";

        /**
         * 文本笔记的Uri
         * 解析为：content://micode_notes/text_note
         */
        public static final Uri CONTENT_URI = Uri.parse("content://" + AUTHORITY + "/text_note");
    }

    // 定义了一个内部类CallNote，继承自DataColumns，表示通话笔记的数据结构
    public static final class CallNote implements DataColumns {
        /**
         * 此记录的通话日期
         * <P> 类型：INTEGER (long) </P>
         */
        public static final String CALL_DATE = DATA1;

        /**
         * 此记录的电话号码
         * <P> 类型：TEXT </P>
         */
        public static final String PHONE_NUMBER = DATA3;

        // 定义了通话笔记的MIME类型
        public static final String CONTENT_TYPE = "vnd.android.cursor.dir/call_note";
        public static final String CONTENT_ITEM_TYPE = "vnd.android.cursor.item/call_note";

        /**
         * 通话笔记的Uri
         * 解析为：content://micode_notes/call_note
         */
        public static final Uri CONTENT_URI = Uri.parse("content://" + AUTHORITY + "/call_note");
    }
}
