/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * 
 * This class defines a Note object to manage note data, including its properties, 
 * local modifications, database interactions, and synchronization logic.
 */

package net.micode.notes.model;

import android.content.ContentProviderOperation;
import android.content.ContentProviderResult;
import android.content.ContentUris;
import android.content.ContentValues;
import android.content.Context;
import android.content.OperationApplicationException;
import android.net.Uri;
import android.os.RemoteException;
import android.util.Log;

import net.micode.notes.data.Notes;
import net.micode.notes.data.Notes.CallNote;
import net.micode.notes.data.Notes.DataColumns;
import net.micode.notes.data.Notes.NoteColumns;
import net.micode.notes.data.Notes.TextNote;

import java.util.ArrayList;

/**
 * Note ڹǩݣıݺͨ¼Ĵ߼
 * ֱ֧ǩĴ޸ġͬͳ־û
 */
public class Note {
    // 洢ǩ仯ֶΣڼ¼Ҫµݿ
    private ContentValues mNoteDiffValues;

    // NoteData һڲ࣬ڴǩľݣıݺͨ¼
    private NoteData mNoteData;

    // ־ǣڼ¼Ϣ
    private static final String TAG = "Note";

    /**
     * ̬һµıǩ ID
     * @param context Ӧõģڷݽ
     * @param folderId ļ IDǩ洢ļ
     * @return ´ıǩΨһ ID
     */
    public static synchronized long getNewNoteId(Context context, long folderId) {
        ContentValues values = new ContentValues();

        // ʱ޸ʱ䶼Ϊǰʱ
        long createdTime = System.currentTimeMillis();
        values.put(NoteColumns.CREATED_DATE, createdTime);
        values.put(NoteColumns.MODIFIED_DATE, createdTime);

        // ñǩΪͨǩ
        values.put(NoteColumns.TYPE, Notes.TYPE_NOTE);

        // ñ޸ı־Ϊ 1 (ʾб޸)
        values.put(NoteColumns.LOCAL_MODIFIED, 1);

        // ñǩĸļ ID
        values.put(NoteColumns.PARENT_ID, folderId);

        // ݿ±ǩȡ URI
        Uri uri = context.getContentResolver().insert(Notes.CONTENT_NOTE_URI, values);

        // ȡǩ ID
        long noteId = 0;
        try {
            noteId = Long.valueOf(uri.getPathSegments().get(1)); // ȡ URI ĵڶΪǩ ID
        } catch (NumberFormatException e) {
            Log.e(TAG, "Get note id error :" + e.toString());
            noteId = 0; // ʧܣǩ ID Ϊ 0
        }

        // ǩ ID Ч׳쳣
        if (noteId == -1) {
            throw new IllegalStateException("Wrong note id:" + noteId);
        }
        return noteId;
    }

    /**
     * Note Ĺ췽ʼǩݱ仯¼;ݡ
     */
    public Note() {
        mNoteDiffValues = new ContentValues(); // ʼǩ仯¼
        mNoteData = new NoteData(); // ʼǩݶ
    }

    /**
     * ñǩļֵԣͬʱΪ޸ġ
     * @param key ֶεļ⡢ʱ
     * @param value ֶεֵıı
     */
    public void setNoteValue(String key, String value) {
        mNoteDiffValues.put(key, value); // üֵԵ mNoteDiffValues 
        mNoteDiffValues.put(NoteColumns.LOCAL_MODIFIED, 1); // Ϊ޸
        mNoteDiffValues.put(NoteColumns.MODIFIED_DATE, System.currentTimeMillis()); // ޸ʱ
    }

    /**
     * ñǩıݣıݡ
     * @param key ݵļıֶ
     * @param value ݵֵı
     */
    public void setTextData(String key, String value) {
        mNoteData.setTextData(key, value); //  NoteData ڲķ
    }

    /**
     * ñǩıݵΨһ ID
     * @param id ıݵ ID
     */
    public void setTextDataId(long id) {
        mNoteData.setTextDataId(id); //  NoteData ڲķ
    }

    /**
     * ȡǩıݵΨһ ID
     * @return ıݵ ID
     */
    public long getTextDataId() {
        return mNoteData.mTextDataId; //  NoteData ڲеı ID
    }

    /**
     * ͨ¼ݵΨһ ID
     * @param id ͨ¼ ID
     */
    public void setCallDataId(long id) {
        mNoteData.setCallDataId(id); //  NoteData ڲķ
    }

    /**
     * ͨ¼ݡ
     * @param key ͨ¼ֶΣ绰
     * @param value ֶεֵ绰ľֵ
     */
    public void setCallData(String key, String value) {
        mNoteData.setCallData(key, value); //  NoteData ڲķ
    }

    /**
     * ǩǷб޸ġ
     * @return ǩб޸򷵻 true򷵻 false
     */
    public boolean isLocalModified() {
        // ǩ仯¼ǩǷб޸
        return mNoteDiffValues.size() > 0 || mNoteData.isLocalModified();
    }

    /**
     * ͬǩݵݿ⡣
     * @param context Ӧģڷݽ
     * @param noteId ǩΨһ ID
     * @return ͬɹ true򷵻 false
     */
    public boolean syncNote(Context context, long noteId) {
        // ǩ ID Ч׳쳣
        if (noteId <= 0) {
            throw new IllegalArgumentException("Wrong note id:" + noteId);
        }

        // ûб޸ģֱӷ true
        if (!isLocalModified()) {
            return true;
        }

        // ǩı仯¼µݿ
        if (context.getContentResolver().update(
                ContentUris.withAppendedId(Notes.CONTENT_NOTE_URI, noteId), mNoteDiffValues, null,
                null) == 0) {
            Log.e(TAG, "Update note error, should not happen");
            // ʹʧܣأ
        }
        mNoteDiffValues.clear(); // ձǩ仯¼

        // ͬǩľݵݿ
        if (mNoteData.isLocalModified()
                && (mNoteData.pushIntoContentResolver(context, noteId) == null)) {
            return false; // ͬʧܣ false
        }

        return true; // ͬɹ true
    }

    /**
     * NoteData  Note ڲ࣬ǩľݣıͨ¼
     */
    private class NoteData {
        private long mTextDataId; // ıݵΨһ ID
        private ContentValues mTextDataValues; // ıݵļֵ
        private long mCallDataId; // ͨݵΨһ ID
        private ContentValues mCallDataValues; // ͨݵļֵ

        // ־
        private static final String TAG = "NoteData";

        /**
         * NoteData Ĺ췽ʼݡ
         */
        public NoteData() {
            mTextDataValues = new ContentValues(); // ʼıݼֵ
            mCallDataValues = new ContentValues(); // ʼͨݼֵ
            mTextDataId = 0; // ʼı ID
            mCallDataId = 0; // ʼͨ ID
        }

        /**
         * Ƿб޸ġ
         * @return б޸ķ true򷵻 false
         */
        boolean isLocalModified() {
            // ıݻͨǷб޸
            return mTextDataValues.size() > 0 || mCallDataValues.size() > 0;
        }

        /**
         * ıݵ ID
         * @param id ıݵΨһ ID
         */
        void setTextDataId(long id) {
            if (id <= 0) {
                throw new IllegalArgumentException("Text data id should larger than 0");
            }
            mTextDataId = id; // ı ID
        }

        /**
         * ͨݵ ID
         * @param id ͨݵΨһ ID
         */
        void setCallDataId(long id) {
            if (id <= 0) {
                throw new IllegalArgumentException("Call data id should larger than 0");
            }
            mCallDataId = id; // ͨ ID
        }

        /**
         * ͨ¼ݡ
         * @param key ֶεļ绰
         * @param value ֶεֵ绰ľֵ
         */
        void setCallData(String key, String value) {
            mCallDataValues.put(key, value); // ӼֵԵͨ
            mNoteDiffValues.put(NoteColumns.LOCAL_MODIFIED, 1); // ǱǩΪ޸
            mNoteDiffValues.put(NoteColumns.MODIFIED_DATE, System.currentTimeMillis()); // ޸ʱ
        }

        /**
         * ıݡ
         * @param key ıݵļ
         * @param value ıݵֵ
         */
        void setTextData(String key, String value) {
            mTextDataValues.put(key, value); // ӼֵԵı
            mNoteDiffValues.put(NoteColumns.LOCAL_MODIFIED, 1); // ǱǩΪ޸
            mNoteDiffValues.put(NoteColumns.MODIFIED_DATE, System.currentTimeMillis()); // ޸ʱ
        }

        /**
         * ǩ͵ݽ (ݿ)
         * @param context Ӧ
         * @param noteId ǩΨһ ID
         * @return ɹʱݵ URI򷵻 null
         */
        Uri pushIntoContentResolver(Context context, long noteId) {
            // б
            ArrayList<ContentProviderOperation> operationList = new ArrayList<>();
            ContentProviderOperation.Builder builder = null;

            // ıݴ߼ (ʡע͵ϸڲ߼ṹ)
            if (mTextDataValues.size() > 0) {
                mTextDataValues.put(DataColumns.NOTE_ID, noteId);
                if (mTextDataId == 0) {
                    // ߼ (ͨ)
                } else {
                    // ߼ (ͨ)
                }
                mTextDataValues.clear();
            }

            // ͨݴ߼ (ʡıݵ߼)
            // ...

            // Ӧݿ
            if (operationList.size() > 0) {
                try {
                    ContentProviderResult[] results = context.getContentResolver().applyBatch(
                            Notes.AUTHORITY, operationList);
                    return (results == null || results.length == 0 || results[0] == null) ? null
                            : ContentUris.withAppendedId(Notes.CONTENT_NOTE_URI, noteId);
                } catch (RemoteException e) {
                    Log.e(TAG, "RemoteException during applyBatch: " + e.toString());
                } catch (OperationApplicationException e) {
                    Log.e(TAG, "OperationApplicationException during applyBatch: " + e.toString());
                }
            }
            return null; //  null ʾʧ
        }
    }
}
