/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.ui; // 定义包名

import android.app.Activity; // 导入Activity类
import android.app.AlertDialog; // 导入AlertDialog类
import android.app.Dialog; // 导入Dialog类
import android.appwidget.AppWidgetManager; // 导入AppWidgetManager类
import android.content.AsyncQueryHandler; // 导入AsyncQueryHandler类
import android.content.ContentResolver; // 导入ContentResolver类
import android.content.ContentValues; // 导入ContentValues类
import android.content.Context; // 导入Context类
import android.content.DialogInterface; // 导入DialogInterface类
import android.content.Intent; // 导入Intent类
import android.content.SharedPreferences; // 导入SharedPreferences类
import android.database.Cursor; // 导入Cursor类
import android.os.AsyncTask; // 导入AsyncTask类
import android.os.Bundle; // 导入Bundle类
import android.preference.PreferenceManager; // 导入PreferenceManager类
import android.text.Editable; // 导入Editable接口
import android.text.TextUtils; // 导入TextUtils类
import android.text.TextWatcher; // 导入TextWatcher接口
import android.util.Log; // 导入Log类
import android.view.ActionMode; // 导入ActionMode类
import android.view.ContextMenu; // 导入ContextMenu类
import android.view.ContextMenu.ContextMenuInfo; // 导入上下文菜单信息类
import android.view.Display; // 导入Display类
import android.view.LayoutInflater; // 导入LayoutInflater类
import android.view.Menu; // 导入Menu类
import android.view.MenuItem; // 导入MenuItem类
import android.view.MenuItem.OnMenuItemClickListener; // 导入菜单项点击监听器接口
import android.view.MotionEvent; // 导入MotionEvent类
import android.view.View; // 导入View类
import android.view.View.OnClickListener; // 导入视图点击监听器接口
import android.view.View.OnCreateContextMenuListener; // 导入创建上下文菜单监听器接口
import android.view.View.OnTouchListener; // 导入触摸监听器接口
import android.view.inputmethod.InputMethodManager; // 导入输入法管理器类
import android.widget.AdapterView; // 导入AdapterView类
import android.widget.AdapterView.OnItemClickListener; // 导入适配器视图项目点击监听器接口
import android.widget.AdapterView.OnItemLongClickListener; // 导入适配器视图项目长按监听器接口
import android.widget.Button; // 导入Button类
import android.widget.EditText; // 导入EditText类
import android.widget.ListView; // 导入ListView类
import android.widget.PopupMenu; // 导入PopupMenu类
import android.widget.TextView; // 导入TextView类
import android.widget.Toast; // 导入Toast类

import net.micode.notes.R; // 导入资源类
import net.micode.notes.data.Notes; // 导入Notes数据类
import net.micode.notes.data.Notes.NoteColumns; // 导入Notes列类
import net.micode.notes.gtask.remote.GTaskSyncService; // 导入GTaskSyncService类
import net.micode.notes.model.WorkingNote; // 导入WorkingNote模型类
import net.micode.notes.tool.BackupUtils; // 导入BackupUtils工具类
import net.micode.notes.tool.DataUtils; // 导入DataUtils工具类
import net.micode.notes.tool.ResourceParser; // 导入ResourceParser解析工具类
import net.micode.notes.ui.NotesListAdapter.AppWidgetAttribute; // 导入小部件属性类
import net.micode.notes.widget.NoteWidgetProvider_2x; // 导入2x笔记小部件提供者
import net.micode.notes.widget.NoteWidgetProvider_4x; // 导入4x笔记小部件提供者

import java.io.BufferedReader; // 导入BufferedReader类
import java.io.IOException; // 导入IOException类
import java.io.InputStream; // 导入InputStream类
import java.io.InputStreamReader; // 导入InputStreamReader类
import java.util.HashSet; // 导入HashSet类

public class NotesListActivity extends Activity implements OnClickListener, OnItemLongClickListener { // 定义NotesListActivity类，继承Activity并实现点击和长按监听接口
    private static final int FOLDER_NOTE_LIST_QUERY_TOKEN = 0; // 文件夹笔记列表查询令牌

    private static final int FOLDER_LIST_QUERY_TOKEN      = 1; // 文件夹列表查询令牌

    private static final int MENU_FOLDER_DELETE = 0; // 文件夹删除菜单项ID

    private static final int MENU_FOLDER_VIEW = 1; // 文件夹查看菜单项ID

    private static final int MENU_FOLDER_CHANGE_NAME = 2; // 文件夹更名菜单项ID

    private static final String PREFERENCE_ADD_INTRODUCTION = "net.micode.notes.introduction"; // 添加介绍的偏好设置键

    private enum ListEditState { // 列表编辑状态枚举
        NOTE_LIST, SUB_FOLDER, CALL_RECORD_FOLDER // 状态: 笔记列表、子文件夹、通话记录文件夹
    };

    private ListEditState mState; // 当前状态

    private BackgroundQueryHandler mBackgroundQueryHandler; // 后台查询处理类

    private NotesListAdapter mNotesListAdapter; // 笔记列表适配器

    private ListView mNotesListView; // 笔记列表视图

    private Button mAddNewNote; // 新增笔记按钮

    private boolean mDispatch; // 是否分发事件的标志

    private int mOriginY; // 事件起始Y坐标

    private int mDispatchY; // 分发事件的Y坐标

    private TextView mTitleBar; // 标题栏文本视图

    private long mCurrentFolderId; // 当前文件夹ID

    private ContentResolver mContentResolver; // 内容解析器

    private ModeCallback mModeCallBack; // 模式回调

    private static final String TAG = "NotesListActivity"; // 日志标签

    public static final int NOTES_LISTVIEW_SCROLL_RATE = 30; // 笔记列表滚动速率

    private NoteItemData mFocusNoteDataItem; // 当前聚焦的笔记数据项

    private static final String NORMAL_SELECTION = NoteColumns.PARENT_ID + "=?"; // 正常查询条件

    private static final String ROOT_FOLDER_SELECTION = "(" + NoteColumns.TYPE + "<>"
            + Notes.TYPE_SYSTEM + " AND " + NoteColumns.PARENT_ID + "=?)" + " OR ("
            + NoteColumns.ID + "=" + Notes.ID_CALL_RECORD_FOLDER + " AND "
            + NoteColumns.NOTES_COUNT + ">0)"; // 根文件夹查询条件

    private final static int REQUEST_CODE_OPEN_NODE = 102; // 打开节点请求代码
    private final static int REQUEST_CODE_NEW_NODE  = 103; // 新增节点请求代码

    @Override
    protected void onCreate(Bundle savedInstanceState) { // 当Activity创建时调用
        super.onCreate(savedInstanceState); // 调用父类的onCreate方法
        setContentView(R.layout.note_list); // 设置内容视图为笔记列表布局
        initResources(); // 初始化资源

        /**
         * 在用户首次使用此应用时插入介绍
         */
        setAppInfoFromRawRes(); // 从原始资源中设置应用信息
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) { // 当Activity的结果回调时调用
        if (resultCode == RESULT_OK
                && (requestCode == REQUEST_CODE_OPEN_NODE || requestCode == REQUEST_CODE_NEW_NODE)) { // 如果结果成功且请求码是打开节点或新建节点
            mNotesListAdapter.changeCursor(null); // 更新适配器的光标
        } else {
            super.onActivityResult(requestCode, resultCode, data); // 调用父类的onActivityResult方法
        }
    }

    private void setAppInfoFromRawRes() { // 从原始资源设置应用信息
        SharedPreferences sp = PreferenceManager.getDefaultSharedPreferences(this); // 获取默认的共享偏好设置
        if (!sp.getBoolean(PREFERENCE_ADD_INTRODUCTION, false)) { // 检查是否已经添加介绍
            StringBuilder sb = new StringBuilder(); // 创建字符串构建器
            InputStream in = null; // 初始化输入流
            try {
                 in = getResources().openRawResource(R.raw.introduction); // 打开原始资源文件
                if (in != null) {
                    InputStreamReader isr = new InputStreamReader(in); // 输入流读取器
                    BufferedReader br = new BufferedReader(isr); // 缓冲读取器
                    char [] buf = new char[1024]; // 创建字符缓冲区
                    int len = 0; // 字符长度初始化
                    while ((len = br.read(buf)) > 0) { // 逐行读取
                        sb.append(buf, 0, len); // 将读取的内容追加到字符串构建器
                    }
                } else {
                    Log.e(TAG, "Read introduction file error"); // 日志输出读取介绍文件错误
                    return; // 返回
                }
            } catch (IOException e) { // 捕获IO异常
                e.printStackTrace(); // 打印异常
                return; // 返回
            } finally { // 在finally中确保关闭输入流
                if(in != null) { // 如果输入流不为空
                    try {
                        in.close(); // 关闭输入流
                    } catch (IOException e) { // 捕获关闭过程中出现的IO异常
                        e.printStackTrace(); // 打印异常
                    }
                }
            }

            WorkingNote note = WorkingNote.createEmptyNote(this, Notes.ID_ROOT_FOLDER, // 创建空笔记
                    AppWidgetManager.INVALID_APPWIDGET_ID, Notes.TYPE_WIDGET_INVALIDE,
                    ResourceParser.RED); // 使用红色背景
            note.setWorkingText(sb.toString()); // 设置工作文本为读取的介绍内容
            if (note.saveNote()) { // 如果保存成功
                sp.edit().putBoolean(PREFERENCE_ADD_INTRODUCTION, true).commit(); // 更新共享偏好设置为已添加介绍
            } else {
                Log.e(TAG, "Save introduction note error"); // 日志输出保存介绍笔记错误
                return; // 返回
            }
        }
    }

    @Override
    protected void onStart() { // 当Activity启动时调用
        super.onStart(); // 调用父类的onStart方法
        startAsyncNotesListQuery(); // 启动异步笔记列表查询
    }

    private void initResources() { // 初始化资源的方法
        mContentResolver = this.getContentResolver(); // 获取内容解析器
        mBackgroundQueryHandler = new BackgroundQueryHandler(this.getContentResolver()); // 初始化后台查询处理器
        mCurrentFolderId = Notes.ID_ROOT_FOLDER; // 初始化当前文件夹ID为根文件夹ID
        mNotesListView = (ListView) findViewById(R.id.notes_list); // 查找笔记列表视图
        mNotesListView.addFooterView(LayoutInflater.from(this).inflate(R.layout.note_list_footer, null), // 添加页脚视图
                null, false);
        mNotesListView.setOnItemClickListener(new OnListItemClickListener()); // 设置列表项点击监听器
        mNotesListView.setOnItemLongClickListener(this); // 设置列表项长按监听器
        mNotesListAdapter = new NotesListAdapter(this); // 创建笔记列表适配器
        mNotesListView.setAdapter(mNotesListAdapter); // 将适配器设置到列表视图
        mAddNewNote = (Button) findViewById(R.id.btn_new_note); // 查找新增笔记按钮
        mAddNewNote.setOnClickListener(this); // 设置新增笔记按钮的点击监听器
        mAddNewNote.setOnTouchListener(new NewNoteOnTouchListener()); // 设置新增笔记按钮的触摸监听器
        mDispatch = false; // 初始化事件分发标志
        mDispatchY = 0; // 初始化Y坐标
        mOriginY = 0; // 初始化起始Y坐标
        mTitleBar = (TextView) findViewById(R.id.tv_title_bar); // 查找标题栏文本视图
        mState = ListEditState.NOTE_LIST; // 初始化为笔记列表状态
        mModeCallBack = new ModeCallback(); // 初始化模式回调
    }

    private class ModeCallback implements ListView.MultiChoiceModeListener, OnMenuItemClickListener { // 定义模式回调类，处理多选择模式和菜单项点击
        private DropdownMenu mDropDownMenu; // 下拉菜单
        private ActionMode mActionMode; // 动作模式
        private MenuItem mMoveMenu; // 移动菜单项

        public boolean onCreateActionMode(ActionMode mode, Menu menu) { // 创建动作模式时调用
            getMenuInflater().inflate(R.menu.note_list_options, menu); // 加载菜单
            menu.findItem(R.id.delete).setOnMenuItemClickListener(this); // 设置删除菜单项的点击监听器
            mMoveMenu = menu.findItem(R.id.move); // 查找移动菜单项
            if (mFocusNoteDataItem.getParentId() == Notes.ID_CALL_RECORD_FOLDER // 判断当前笔记的父ID是否为通话记录文件夹
                    || DataUtils.getUserFolderCount(mContentResolver) == 0) {
                mMoveMenu.setVisible(false); // 若是，则隐藏移动菜单项
            } else {
                mMoveMenu.setVisible(true); // 否则，显示移动菜单项
                mMoveMenu.setOnMenuItemClickListener(this); // 设置移动菜单项的点击监听器
            }
            mActionMode = mode; // 设置当前动作模式
            mNotesListAdapter.setChoiceMode(true); // 启用适配器的选择模式
            mNotesListView.setLongClickable(false); // 禁用列表长按
            mAddNewNote.setVisibility(View.GONE); // 隐藏新增笔记按钮

            View customView = LayoutInflater.from(NotesListActivity.this).inflate( // 创建自定义视图
                    R.layout.note_list_dropdown_menu, null);
            mode.setCustomView(customView); // 设置自定义视图
            mDropDownMenu = new DropdownMenu(NotesListActivity.this, // 创建下拉菜单
                    (Button) customView.findViewById(R.id.selection_menu),
                    R.menu.note_list_dropdown);
            mDropDownMenu.setOnDropdownMenuItemClickListener(new PopupMenu.OnMenuItemClickListener(){ // 设置下拉菜单项点击监听器
                public boolean onMenuItemClick(MenuItem item) {
                    mNotesListAdapter.selectAll(!mNotesListAdapter.isAllSelected()); // 切换全选状态
                    updateMenu(); // 更新菜单
                    return true; // 返回true，表示事件已处理
                }

            });
            return true; // 返回true，表示创建成功
        }

        private void updateMenu() { // 更新菜单的方法
            int selectedCount = mNotesListAdapter.getSelectedCount(); // 获取选中项数量
            // 更新下拉菜单
            String format = getResources().getString(R.string.menu_select_title, selectedCount); // 格式化标题
            mDropDownMenu.setTitle(format); // 设置下拉菜单标题
            MenuItem item = mDropDownMenu.findItem(R.id.action_select_all); // 查找全选菜单项
            if (item != null) {
                if (mNotesListAdapter.isAllSelected()) { // 如果全选状态为真
                    item.setChecked(true); // 设置为已选中
                    item.setTitle(R.string.menu_deselect_all); // 更新标题为取消全选
                } else {
                    item.setChecked(false); // 否则设置为未选中
                    item.setTitle(R.string.menu_select_all); // 更新标题为全选
                }
            }
        }

        public boolean onPrepareActionMode(ActionMode mode, Menu menu) { // 准备动作模式时调用
            return false; // 默认返回false，表示不改变菜单
        }

        public boolean onActionItemClicked(ActionMode mode, MenuItem item) { // 动作模式中菜单项点击事件
            return false; // 默认返回false
        }

        public void onDestroyActionMode(ActionMode mode) { // 动作模式销毁时调用
            mNotesListAdapter.setChoiceMode(false); // 关闭适配器选择模式
            mNotesListView.setLongClickable(true); // 启用长按
            mAddNewNote.setVisibility(View.VISIBLE); // 显示新增笔记按钮
        }

        public void finishActionMode() { // 结束动作模式
            mActionMode.finish(); // 结束当前动作模式
        }

        public void onItemCheckedStateChanged(ActionMode mode, int position, long id, boolean checked) { // 选中状态变化时调用
            mNotesListAdapter.setCheckedItem(position, checked); // 更新选中项状态
            updateMenu(); // 更新菜单
        }

        public boolean onMenuItemClick(MenuItem item) { // 菜单项点击事件
            if (mNotesListAdapter.getSelectedCount() == 0) { // 如果没有选中项
                Toast.makeText(NotesListActivity.this, getString(R.string.menu_select_none), // 显示提示
                        Toast.LENGTH_SHORT).show();
                return true; // 返回true，表示事件已处理
            }

            switch (item.getItemId()) { // 根据菜单项ID处理事件
                case R.id.delete: // 删除菜单项
                    AlertDialog.Builder builder = new AlertDialog.Builder(NotesListActivity.this); // 创建删除确认对话框
                    builder.setTitle(getString(R.string.alert_title_delete)); // 设置对话框标题
                    builder.setIcon(android.R.drawable.ic_dialog_alert); // 设置对话框图标
                    builder.setMessage(getString(R.string.alert_message_delete_notes, // 设置对话框消息
                                             mNotesListAdapter.getSelectedCount()));
                    builder.setPositiveButton(android.R.string.ok, // 点击确认按钮时的事件
                                             new DialogInterface.OnClickListener() {
                                                 public void onClick(DialogInterface dialog, int which) {
                                                     batchDelete(); // 批量删除选中的笔记
                                                 }
                                             });
                    builder.setNegativeButton(android.R.string.cancel, null); // 设置取消按钮
                    builder.show(); // 显示对话框
                    break;
                case R.id.move: // 移动菜单项
                    startQueryDestinationFolders(); // 查询可移动的文件夹
                    break;
                default:
                    return false; // 默认返回false，表示未处理
            }
            return true; // 返回true，表示菜单项已处理
        }
    }

    private class NewNoteOnTouchListener implements OnTouchListener { // 新增笔记按钮的触摸监听器

        public boolean onTouch(View v, MotionEvent event) { // 触摸事件处理
            switch (event.getAction()) { // 根据触摸事件类型进行处理
                case MotionEvent.ACTION_DOWN: { // 按下事件
                    Display display = getWindowManager().getDefaultDisplay(); // 获取窗口显示
                    int screenHeight = display.getHeight(); // 获取屏幕高度
                    int newNoteViewHeight = mAddNewNote.getHeight(); // 获取新增笔记按钮的高度
                    int start = screenHeight - newNoteViewHeight; // 计算按钮起始位置
                    int eventY = start + (int) event.getY(); // 事件Y坐标
                    /**
                     * 减去标题栏高度
                     */
                    if (mState == ListEditState.SUB_FOLDER) { // 如果当前状态为子文件夹
                        eventY -= mTitleBar.getHeight(); // 在Y坐标上减去标题栏高度
                        start -= mTitleBar.getHeight();
                    }
                    /**
                     * HACKME: 当点击“新增笔记”按钮的透明部分时，将事件分发给
                     * 后面的列表视图。此按钮的透明部分可以用公式y=-0.12x+94(
                     * 单位:像素)表示，并且这条线就是按钮的顶部。
                     * 坐标基于“新增笔记”按钮的左边界。94表示透明部分的最大高度。
                     * 注意，如果按钮的背景变化，这个公式也会变化。这是非常糟糕的，只是为了
                     * UI设计师的强烈要求。
                     */
                    if (event.getY() < (event.getX() * (-0.12) + 94)) { // 如果触摸点在透明区域
                        View view = mNotesListView.getChildAt(mNotesListView.getChildCount() - 1 // 获取列表最后一个子视图
                                - mNotesListView.getFooterViewsCount());
                        if (view != null && view.getBottom() > start // 检查触摸点是否在列表最底部的范围内
                                && (view.getTop() < (start + 94))) {
                            mOriginY = (int) event.getY(); // 记录触摸事件的Y坐标
                            mDispatchY = eventY; // 更新分发Y坐标
                            event.setLocation(event.getX(), mDispatchY); // 设置触摸事件位置
                            mDispatch = true; // 将分发标志设置为true
                            return mNotesListView.dispatchTouchEvent(event); // 将事件分发给列表视图
                        }
                    }
                    break;
                }
                case MotionEvent.ACTION_MOVE: { // 移动事件
                    if (mDispatch) { // 如果分发标志为true
                        mDispatchY += (int) event.getY() - mOriginY; // 更新分发Y坐标
                        event.setLocation(event.getX(), mDispatchY); // 设置触摸事件位置
                        return mNotesListView.dispatchTouchEvent(event); // 将事件分发给列表视图
                    }
                    break;
                }
                default: { // 默认事件处理
                    if (mDispatch) { // 如果分发标志为true
                        event.setLocation(event.getX(), mDispatchY); // 设置触摸事件位置
                        mDispatch = false; // 重置分发标志
                        return mNotesListView.dispatchTouchEvent(event); // 将事件分发给列表视图
                    }
                    break;
                }
            }
            return false; // 返回false，表示未处理
        }

    };

    private void startAsyncNotesListQuery() { // 启动异步笔记列表查询的方法
        String selection = (mCurrentFolderId == Notes.ID_ROOT_FOLDER) ? ROOT_FOLDER_SELECTION // 根据当前文件夹ID选择查询条件
                : NORMAL_SELECTION;
        mBackgroundQueryHandler.startQuery(FOLDER_NOTE_LIST_QUERY_TOKEN, // 启动查询
                null,
                Notes.CONTENT_NOTE_URI, // 查询URI
                NoteItemData.PROJECTION, // 查询的列
                selection, // 查询条件
                new String[] { // 查询参数
                        String.valueOf(mCurrentFolderId)
                },
                NoteColumns.TYPE + " DESC," + NoteColumns.MODIFIED_DATE + " DESC"); // 排序条件
    }

    private final class BackgroundQueryHandler extends AsyncQueryHandler { // 定义后台查询处理器类
        public BackgroundQueryHandler(ContentResolver contentResolver) {
            super(contentResolver); // 调用父类构造函数
        }

        @Override
        protected void onQueryComplete(int token, Object cookie, Cursor cursor) { // 查询完成时调用
            switch (token) { // 根据查询令牌处理
                case FOLDER_NOTE_LIST_QUERY_TOKEN:
                    mNotesListAdapter.changeCursor(cursor); // 更新适配器的光标
                    break;
                case FOLDER_LIST_QUERY_TOKEN:
                    if (cursor != null && cursor.getCount() > 0) { // 如果游标不为空且有结果
                        showFolderListMenu(cursor); // 显示文件夹列表菜单
                    } else {
                        Log.e(TAG, "Query folder failed"); // 日志错误输出
                    }
                    break;
                default:
                    return; // 默认返回
            }
        }
    }

    private void showFolderListMenu(Cursor cursor) { // 显示文件夹列表菜单的方法
        AlertDialog.Builder builder = new AlertDialog.Builder(NotesListActivity.this); // 创建对话框构建器
        builder.setTitle(R.string.menu_title_select_folder); // 设置对话框标题
        final FoldersListAdapter adapter = new FoldersListAdapter(this, cursor); // 创建文件夹列表适配器
        builder.setAdapter(adapter, new DialogInterface.OnClickListener() { // 设置适配器和点击监听器
            public void onClick(DialogInterface dialog, int which) { // 对话框项目点击事件
                DataUtils.batchMoveToFolder(mContentResolver, // 批量移动已选笔记到目标文件夹
                        mNotesListAdapter.getSelectedItemIds(), adapter.getItemId(which));
                Toast.makeText( // 显示成功提示
                        NotesListActivity.this,
                        getString(R.string.format_move_notes_to_folder,
                                mNotesListAdapter.getSelectedCount(),
                                adapter.getFolderName(NotesListActivity.this, which)),
                        Toast.LENGTH_SHORT).show();
                mModeCallBack.finishActionMode(); // 结束模式回调
            }
        });
        builder.show(); // 显示对话框
    }

    private void createNewNote() { // 创建新笔记的方法
        Intent intent = new Intent(this, NoteEditActivity.class); // 创建意图
        intent.setAction(Intent.ACTION_INSERT_OR_EDIT); // 设置操作为插入或编辑
        intent.putExtra(Notes.INTENT_EXTRA_FOLDER_ID, mCurrentFolderId); // 添加当前文件夹ID到意图
        this.startActivityForResult(intent, REQUEST_CODE_NEW_NODE); // 启动新Activity并请求结果
    }

    private void batchDelete() { // 批量删除的方法
        new AsyncTask<Void, Void, HashSet<AppWidgetAttribute>>() { // 创建异步任务
            protected HashSet<AppWidgetAttribute> doInBackground(Void... unused) { // 背景线程执行的任务
                HashSet<AppWidgetAttribute> widgets = mNotesListAdapter.getSelectedWidget(); // 获取已选中的小部件
                if (!isSyncMode()) { // 如果不在同步模式
                    // 不在同步模式时，直接删除笔记
                    if (DataUtils.batchDeleteNotes(mContentResolver, mNotesListAdapter
                            .getSelectedItemIds())) {
                    } else {
                        Log.e(TAG, "Delete Note error"); // 日志输出删除笔记错误
                    }
                } else { // 在同步模式
                    // 在同步模式下，将删除的笔记移动到回收站文件夹
                    if (!DataUtils.batchMoveToFolder(mContentResolver, mNotesListAdapter
                            .getSelectedItemIds(), Notes.ID_TRASH_FOLER)) { // 移动到回收站
                        Log.e(TAG, "Move notes to trash folder error, should not happens"); // 日志输出错误信息
                    }
                }
                return widgets; // 返回小部件集合
            }

            @Override
            protected void onPostExecute(HashSet<AppWidgetAttribute> widgets) { // 任务完成后调用
                if (widgets != null) { // 如果小部件不为空
                    for (AppWidgetAttribute widget : widgets) { // 遍历小部件集合
                        if (widget.widgetId != AppWidgetManager.INVALID_APPWIDGET_ID // 检查小部件ID是否有效
                                && widget.widgetType != Notes.TYPE_WIDGET_INVALIDE) { // 检查小部件类型是否有效
                            updateWidget(widget.widgetId, widget.widgetType); // 更新小部件
                        }
                    }
                }
                mModeCallBack.finishActionMode(); // 结束模式回调
            }
        }.execute(); // 执行异步任务
    }

    private void deleteFolder(long folderId) { // 删除指定文件夹的方法
        if (folderId == Notes.ID_ROOT_FOLDER) { // 如果是根文件夹ID
            Log.e(TAG, "Wrong folder id, should not happen " + folderId); // 日志错误输出
            return; // 返回
        }

        HashSet<Long> ids = new HashSet<Long>(); // 创建ID集合
        ids.add(folderId); // 将文件夹ID添加到集合
        HashSet<AppWidgetAttribute> widgets = DataUtils.getFolderNoteWidget(mContentResolver,
                folderId); // 获取该文件夹相关的小部件
        if (!isSyncMode()) { // 如果不在同步模式
            // 如果不在同步模式，直接删除文件夹
            DataUtils.batchDeleteNotes(mContentResolver, ids); // 批量删除笔记
        } else { // 在同步模式
            // 在同步模式下，将删除的文件夹移动到回收站
            DataUtils.batchMoveToFolder(mContentResolver, ids, Notes.ID_TRASH_FOLER);
        }
        if (widgets != null) { // 如果小部件不为空
            for (AppWidgetAttribute widget : widgets) { // 遍历小部件集合
                if (widget.widgetId != AppWidgetManager.INVALID_APPWIDGET_ID // 检查小部件ID是否有效
                        && widget.widgetType != Notes.TYPE_WIDGET_INVALIDE) { // 检查小部件类型是否有效
                    updateWidget(widget.widgetId, widget.widgetType); // 更新小部件
                }
            }
        }
    }

    private void openNode(NoteItemData data) { // 打开指定笔记的方法
        Intent intent = new Intent(this, NoteEditActivity.class); // 创建打开笔记的意图
        intent.setAction(Intent.ACTION_VIEW); // 设置操作为查看
        intent.putExtra(Intent.EXTRA_UID, data.getId()); // 添加笔记ID到意图
        this.startActivityForResult(intent, REQUEST_CODE_OPEN_NODE); // 启动笔记编辑Activity并请求结果
    }

    private void openFolder(NoteItemData data) { // 打开指定文件夹的方法
        mCurrentFolderId = data.getId(); // 设置当前文件夹ID
        startAsyncNotesListQuery(); // 启动异步笔记列表查询
        if (data.getId() == Notes.ID_CALL_RECORD_FOLDER) { // 如果是通话记录文件夹
            mState = ListEditState.CALL_RECORD_FOLDER; // 更新状态为通话记录
            mAddNewNote.setVisibility(View.GONE); // 隐藏新增笔记按钮
        } else {
            mState = ListEditState.SUB_FOLDER; // 更新状态为子文件夹
        }
        if (data.getId() == Notes.ID_CALL_RECORD_FOLDER) { // 如果是通话记录文件夹
            mTitleBar.setText(R.string.call_record_folder_name); // 设置标题为通话记录文件夹名称
        } else {
            mTitleBar.setText(data.getSnippet()); // 设置标题为文件夹的摘要
        }
        mTitleBar.setVisibility(View.VISIBLE); // 显示标题栏
    }

    public void onClick(View v) { // 按钮点击处理
        switch (v.getId()) { // 根据视图ID处理不同的点击事件
            case R.id.btn_new_note: // 新增笔记按钮
                createNewNote(); // 创建新笔记
                break;
            default:
                break; // 默认情况下不处理
        }
    }

    private void showSoftInput() { // 显示软输入法的方法
        InputMethodManager inputMethodManager = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE); // 获取输入法管理器
        if (inputMethodManager != null) { // 如果输入法管理器不为null
            inputMethodManager.toggleSoftInput(InputMethodManager.SHOW_FORCED, 0); // 切换软输入法显示
        }
    }

    private void hideSoftInput(View view) { // 隐藏软输入法的方法
        InputMethodManager inputMethodManager = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE); // 获取输入法管理器
        inputMethodManager.hideSoftInputFromWindow(view.getWindowToken(), 0); // 隐藏软输入法
    }

    private void showCreateOrModifyFolderDialog(final boolean create) { // 显示创建或修改文件夹对话框的方法
        final AlertDialog.Builder builder = new AlertDialog.Builder(this); // 创建对话框构建器
        View view = LayoutInflater.from(this).inflate(R.layout.dialog_edit_text, null); // 加载对话框布局
        final EditText etName = (EditText) view.findViewById(R.id.et_foler_name); // 获取文件夹名称输入框
        showSoftInput(); // 显示软输入法
        if (!create) { // 如果是修改操作
            if (mFocusNoteDataItem != null) { // 如果当前聚焦的笔记数据项不为null
                etName.setText(mFocusNoteDataItem.getSnippet()); // 设置输入框文本为当前笔记摘要
                builder.setTitle(getString(R.string.menu_folder_change_name)); // 设置对话框标题为修改名称
            } else {
                Log.e(TAG, "The long click data item is null"); // 日志输出错误信息
                return; // 返回
            }
        } else {
            etName.setText(""); // 清空输入框
            builder.setTitle(this.getString(R.string.menu_create_folder)); // 设置对话框标题为创建文件夹
        }

        builder.setPositiveButton(android.R.string.ok, null); // 设置确认按钮
        builder.setNegativeButton(android.R.string.cancel, new DialogInterface.OnClickListener() { // 设置取消按钮
            public void onClick(DialogInterface dialog, int which) {
                hideSoftInput(etName); // 隐藏软输入法
            }
        });

        final Dialog dialog = builder.setView(view).show(); // 显示对话框
        final Button positive = (Button)dialog.findViewById(android.R.id.button1); // 获取确认按钮
        positive.setOnClickListener(new OnClickListener() { // 设置确认按钮的点击事件
            public void onClick(View v) {
                hideSoftInput(etName); // 隐藏软输入法
                String name = etName.getText().toString(); // 获取输入框文本
                if (DataUtils.checkVisibleFolderName(mContentResolver, name)) { // 检查文件夹名称是否存在
                    Toast.makeText(NotesListActivity.this, getString(R.string.folder_exist, name),
                            Toast.LENGTH_LONG).show(); // 显示文件夹已存在的提示
                    etName.setSelection(0, etName.length()); // 设置焦点到输入框
                    return; // 返回
                }
                if (!create) { // 如果是修改文件夹名称
                    if (!TextUtils.isEmpty(name)) { // 如果输入框不为空
                        ContentValues values = new ContentValues(); // 创建ContentValues
                        values.put(NoteColumns.SNIPPET, name); // 添加名称
                        values.put(NoteColumns.TYPE, Notes.TYPE_FOLDER); // 添加类型
                        values.put(NoteColumns.LOCAL_MODIFIED, 1); // 标记为本地修改
                        mContentResolver.update(Notes.CONTENT_NOTE_URI, values, NoteColumns.ID // 更新文件夹信息
                                + "=?", new String[] {
                            String.valueOf(mFocusNoteDataItem.getId()) // 用当前文件夹ID更新
                        });
                    }
                } else if (!TextUtils.isEmpty(name)) { // 如果是创建新文件夹且输入框不为空
                    ContentValues values = new ContentValues(); // 创建ContentValues
                    values.put(NoteColumns.SNIPPET, name); // 添加文件夹名称
                    values.put(NoteColumns.TYPE, Notes.TYPE_FOLDER); // 设置类型为文件夹
                    mContentResolver.insert(Notes.CONTENT_NOTE_URI, values); // 插入新文件夹到数据库
                }
                dialog.dismiss(); // 关闭对话框
            }
        });

        if (TextUtils.isEmpty(etName.getText())) { // 如果输入框没有内容
            positive.setEnabled(false); // 禁用确认按钮
        }
        /**
         * 当名称编辑框为空时，禁用确认按钮
         */
        etName.addTextChangedListener(new TextWatcher() { // 添加文本变化监听器
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {
                // 不处理
            }

            public void onTextChanged(CharSequence s, int start, int before, int count) { // 文本变化时调用
                if (TextUtils.isEmpty(etName.getText())) { // 如果输入框为空
                    positive.setEnabled(false); // 禁用确认按钮
                } else {
                    positive.setEnabled(true); // 否则启用确认按钮
                }
            }

            public void afterTextChanged(Editable s) {
                // 不处理
            }
        });
    }

    @Override
    public void onBackPressed() { // 当按下返回按钮时调用
        switch (mState) { // 根据当前状态处理
            case SUB_FOLDER: // 如果为子文件夹状态
                mCurrentFolderId = Notes.ID_ROOT_FOLDER; // 设置当前文件夹ID为根文件夹ID
                mState = ListEditState.NOTE_LIST; // 更新状态为笔记列表
                startAsyncNotesListQuery(); // 启动异步笔记列表查询
                mTitleBar.setVisibility(View.GONE); // 隐藏标题栏
                break;
            case CALL_RECORD_FOLDER: // 如果为通话记录文件夹状态
                mCurrentFolderId = Notes.ID_ROOT_FOLDER; // 设置当前文件夹ID为根文件夹ID
                mState = ListEditState.NOTE_LIST; // 更新状态为笔记列表
                mAddNewNote.setVisibility(View.VISIBLE); // 显示新增笔记按钮
                mTitleBar.setVisibility(View.GONE); // 隐藏标题栏
                startAsyncNotesListQuery(); // 启动异步笔记列表查询
                break;
            case NOTE_LIST: // 如果为笔记列表状态
                super.onBackPressed(); // 调用父类返回
                break;
            default:
                break; // 默认情况不处理
        }
    }

    private void updateWidget(int appWidgetId, int appWidgetType) { // 更新小部件的方法
        Intent intent = new Intent(AppWidgetManager.ACTION_APPWIDGET_UPDATE); // 创建更新小部件的Intent
        if (appWidgetType == Notes.TYPE_WIDGET_2X) { // 如果小部件类型为2x
            intent.setClass(this, NoteWidgetProvider_2x.class); // 设置为2x小部件提供者
        } else if (appWidgetType == Notes.TYPE_WIDGET_4X) { // 如果小部件类型为4x
            intent.setClass(this, NoteWidgetProvider_4x.class); // 设置为4x小部件提供者
        } else {
            Log.e(TAG, "Unspported widget type"); // 日志输出不支持的小部件类型
            return; // 返回
        }

        intent.putExtra(AppWidgetManager.EXTRA_APPWIDGET_IDS, new int[] { // 将小部件ID放入Intent中
            appWidgetId
        });

        sendBroadcast(intent); // 发送广播以更新小部件
        setResult(RESULT_OK, intent); // 设置结果为成功
    }

    private final OnCreateContextMenuListener mFolderOnCreateContextMenuListener = new OnCreateContextMenuListener() { // 创建文件夹上下文菜单的监听器
        public void onCreateContextMenu(ContextMenu menu, View v, ContextMenuInfo menuInfo) { // 创建上下文菜单时调用
            if (mFocusNoteDataItem != null) { // 如果当前聚焦的笔记数据项不为null
                menu.setHeaderTitle(mFocusNoteDataItem.getSnippet()); // 设置菜单标题为当前笔记摘要
                menu.add(0, MENU_FOLDER_VIEW, 0, R.string.menu_folder_view); // 添加查看文件夹菜单项
                menu.add(0, MENU_FOLDER_DELETE, 0, R.string.menu_folder_delete); // 添加删除文件夹菜单项
                menu.add(0, MENU_FOLDER_CHANGE_NAME, 0, R.string.menu_folder_change_name); // 添加更改文件夹名称菜单项
            }
        }
    };

    @Override
    public void onContextMenuClosed(Menu menu) { // 当上下文菜单关闭时调用
        if (mNotesListView != null) { // 如果笔记列表视图不为空
            mNotesListView.setOnCreateContextMenuListener(null); // 清除上下文菜单创建监听器
        }
        super.onContextMenuClosed(menu); // 调用父类的处理方法
    }

    @Override
    public boolean onContextItemSelected(MenuItem item) { // 当上下文菜单项被选中时调用
        if (mFocusNoteDataItem == null) { // 如果当前聚焦的笔记数据项为空
            Log.e(TAG, "The long click data item is null"); // 日志输出错误信息
            return false; // 返回false
        }
        switch (item.getItemId()) { // 根据菜单项ID处理操作
            case MENU_FOLDER_VIEW: // 查看文件夹菜单项
                openFolder(mFocusNoteDataItem); // 打开文件夹
                break;
            case MENU_FOLDER_DELETE: // 删除文件夹菜单项
                AlertDialog.Builder builder = new AlertDialog.Builder(this); // 创建删除确认对话框
                builder.setTitle(getString(R.string.alert_title_delete)); // 设置对话框标题
                builder.setIcon(android.R.drawable.ic_dialog_alert); // 设置对话框图标
                builder.setMessage(getString(R.string.alert_message_delete_folder)); // 设置对话框消息
                builder.setPositiveButton(android.R.string.ok, // 设置确认按钮
                        new DialogInterface.OnClickListener() {
                            public void onClick(DialogInterface dialog, int which) {
                                deleteFolder(mFocusNoteDataItem.getId()); // 删除当前聚焦的文件夹
                            }
                        });
                builder.setNegativeButton(android.R.string.cancel, null); // 设置取消按钮
                builder.show(); // 显示对话框
                break;
            case MENU_FOLDER_CHANGE_NAME: // 更改文件夹名称菜单项
                showCreateOrModifyFolderDialog(false); // 显示修改文件夹对话框
                break;
            default:
                break; // 默认不处理
        }

        return true; // 返回true，表示操作已处理
    }

    @Override
    public boolean onPrepareOptionsMenu(Menu menu) { // 准备选项菜单时调用
        menu.clear(); // 清空菜单
        if (mState == ListEditState.NOTE_LIST) { // 如果当前状态是笔记列表
            getMenuInflater().inflate(R.menu.note_list, menu); // 加载笔记列表菜单
            // 设置同步或取消同步
            menu.findItem(R.id.menu_sync).setTitle(
                    GTaskSyncService.isSyncing() ? R.string.menu_sync_cancel : R.string.menu_sync);
        } else if (mState == ListEditState.SUB_FOLDER) { // 如果当前状态是子文件夹
            getMenuInflater().inflate(R.menu.sub_folder, menu); // 加载子文件夹菜单
        } else if (mState == ListEditState.CALL_RECORD_FOLDER) { // 如果当前状态是通话记录文件夹
            getMenuInflater().inflate(R.menu.call_record_folder, menu); // 加载通话记录文件夹菜单
        } else {
            Log.e(TAG, "Wrong state:" + mState); // 日志输出错误状态
        }
        return true; // 返回true，表示菜单准备成功
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) { // 处理选项菜单项选择的事件
        switch (item.getItemId()) { // 根据菜单项的ID执行相应操作
            case R.id.menu_new_folder: { // 新建文件夹菜单项
                showCreateOrModifyFolderDialog(true); // 显示创建文件夹对话框
                break;
            }
            case R.id.menu_export_text: { // 导出文本菜单项
                exportNoteToText(); // 导出笔记为文本
                break;
            }
            case R.id.menu_sync: { // 同步菜单项
                if (isSyncMode()) { // 如果是同步模式
                    if (TextUtils.equals(item.getTitle(), getString(R.string.menu_sync))) { // 如果菜单标题为同步
                        GTaskSyncService.startSync(this); // 启动同步服务
                    } else {
                        GTaskSyncService.cancelSync(this); // 取消同步
                    }
                } else {
                    startPreferenceActivity(); // 启动设置Activity
                }
                break;
            }
            case R.id.menu_setting: { // 设置菜单项
                startPreferenceActivity(); // 启动设置Activity
                break;
            }
            case R.id.menu_new_note: { // 新建笔记菜单项
                createNewNote(); // 创建新笔记
                break;
            }
            case R.id.menu_search: // 搜索菜单项
                onSearchRequested(); // 请求搜索
                break;
            default:
                break; // 默认不处理
        }
        return true; // 返回true，表示菜单项已处理
    }

    @Override
    public boolean onSearchRequested() { // 当搜索请求时被调用
        startSearch(null, false, null /* appData */, false); // 启动搜索
        return true; // 返回true，表示事件已处理
    }

    private void exportNoteToText() { // 导出笔记为文本的方法
        final BackupUtils backup = BackupUtils.getInstance(NotesListActivity.this); // 获取备份工具实例
        new AsyncTask<Void, Void, Integer>() { // 创建异步任务

            @Override
            protected Integer doInBackground(Void... unused) { // 在后台执行的任务
                return backup.exportToText(); // 调用导出方法
            }

            @Override
            protected void onPostExecute(Integer result) { // 任务执行完毕后调用
                if (result == BackupUtils.STATE_SD_CARD_UNMOUONTED) { // 如果SD卡未挂载
                    AlertDialog.Builder builder = new AlertDialog.Builder(NotesListActivity.this); // 创建警告对话框
                    builder.setTitle(NotesListActivity.this
                            .getString(R.string.failed_sdcard_export)); // 设置对话框标题
                    builder.setMessage(NotesListActivity.this
                            .getString(R.string.error_sdcard_unmounted)); // 设置对话框消息
                    builder.setPositiveButton(android.R.string.ok, null); // 设置确认按钮
                    builder.show(); // 显示对话框
                } else if (result == BackupUtils.STATE_SUCCESS) { // 如果导出成功
                    AlertDialog.Builder builder = new AlertDialog.Builder(NotesListActivity.this); // 创建成功对话框
                    builder.setTitle(NotesListActivity.this
                            .getString(R.string.success_sdcard_export)); // 设置标题
                    builder.setMessage(NotesListActivity.this.getString(
                            R.string.format_exported_file_location, backup
                                    .getExportedTextFileName(), backup.getExportedTextFileDir())); // 设置导出文件位置消息
                    builder.setPositiveButton(android.R.string.ok, null); // 设置确认按钮
                    builder.show(); // 显示对话框
                } else if (result == BackupUtils.STATE_SYSTEM_ERROR) { // 如果系统错误
                    AlertDialog.Builder builder = new AlertDialog.Builder(NotesListActivity.this); // 创建错误对话框
                    builder.setTitle(NotesListActivity.this
                            .getString(R.string.failed_sdcard_export)); // 设置标题
                    builder.setMessage(NotesListActivity.this
                            .getString(R.string.error_sdcard_export)); // 设置消息
                    builder.setPositiveButton(android.R.string.ok, null); // 设置确认按钮
                    builder.show(); // 显示对话框
                }
            }

        }.execute(); // 执行异步任务
    }

    private boolean isSyncMode() { // 判断是否在同步模式的方法
        return NotesPreferenceActivity.getSyncAccountName(this).trim().length() > 0; // 如果同步账户名称不为空则返回true
    }

    private void startPreferenceActivity() { // 启动设置Activity的方法
        Activity from = getParent() != null ? getParent() : this; // 获取父Activity，如果没有则使用当前Activity
        Intent intent = new Intent(from, NotesPreferenceActivity.class); // 创建意图
        from.startActivityIfNeeded(intent, -1); // 启动设置Activity
    }

    private class OnListItemClickListener implements OnItemClickListener { // 定义内部类 OnListItemClickListener，实现列表项点击监听器

    public void onItemClick(AdapterView<?> parent, View view, int position, long id) { // 处理点击事件
        if (view instanceof NotesListItem) { // 检查点击的视图是否为 NotesListItem 实例
            NoteItemData item = ((NotesListItem) view).getItemData(); // 获取点击的项数据

            if (mNotesListAdapter.isInChoiceMode()) { // 如果适配器处于选择模式
                if (item.getType() == Notes.TYPE_NOTE) { // 如果点击的项是笔记类型
                    position = position - mNotesListView.getHeaderViewsCount(); // 调整位置，以考虑头部视图的数量
                    mModeCallBack.onItemCheckedStateChanged(null, position, id, // 通知模式回调选中状态变化
                            !mNotesListAdapter.isSelectedItem(position)); // 设置选中状态为相反
                }
                return; // 返回，不再处理其他情况
            }

            switch (mState) { // 根据当前状态处理不同的逻辑
                case NOTE_LIST: // 笔记列表状态
                    if (item.getType() == Notes.TYPE_FOLDER // 如果为文件夹类型
                            || item.getType() == Notes.TYPE_SYSTEM) { // 或者是系统类型
                        openFolder(item); // 打开文件夹
                    } else if (item.getType() == Notes.TYPE_NOTE) { // 如果是笔记类型
                        openNode(item); // 打开笔记
                    } else {
                        Log.e(TAG, "Wrong note type in NOTE_LIST"); // 日志输出错误信息
                    }
                    break;
                case SUB_FOLDER: // 子文件夹状态
                case CALL_RECORD_FOLDER: // 通话记录文件夹状态
                    if (item.getType() == Notes.TYPE_NOTE) { // 如果是笔记类型
                        openNode(item); // 打开笔记
                    } else {
                        Log.e(TAG, "Wrong note type in SUB_FOLDER"); // 日志输出错误信息
                    }
                    break;
                default: // 默认情况
                    break;
            }
        }
    }

}

private void startQueryDestinationFolders() { // 启动查询目标文件夹的方法
    // 定义选择条件，选择类型为文件夹的项且父ID不等于当前文件夹ID，且ID不等于通话记录文件夹ID 
    String selection = NoteColumns.TYPE + "=? AND " + NoteColumns.PARENT_ID + "<>? AND " + NoteColumns.ID + "<>?";
    selection = (mState == ListEditState.NOTE_LIST) ? selection : // 如果当前状态是笔记列表
        "(" + selection + ") OR (" + NoteColumns.ID + "=" + Notes.ID_ROOT_FOLDER + ")"; // 否则也查询根文件夹

    mBackgroundQueryHandler.startQuery(FOLDER_LIST_QUERY_TOKEN, // 启动异步查询
            null,
            Notes.CONTENT_NOTE_URI, // 查询的URI
            FoldersListAdapter.PROJECTION, // 查询的列
            selection, // 查询条件
            new String[] { // 查询参数
                String.valueOf(Notes.TYPE_FOLDER), // 当前选择的文件夹
                String.valueOf(Notes.ID_TRASH_FOLER), // 回收站文件夹ID
                String.valueOf(mCurrentFolderId) // 当前文件夹ID
            },
            NoteColumns.MODIFIED_DATE + " DESC"); // 按修改日期降序排序
}

public boolean onItemLongClick(AdapterView<?> parent, View view, int position, long id) { // 处理长按事件
    if (view instanceof NotesListItem) { // 如果视图是 NotesListItem 实例
        mFocusNoteDataItem = ((NotesListItem) view).getItemData(); // 获取当前聚焦的笔记数据项
        if (mFocusNoteDataItem.getType() == Notes.TYPE_NOTE && !mNotesListAdapter.isInChoiceMode()) { // 如果为笔记类型且不在选择模式
            if (mNotesListView.startActionMode(mModeCallBack) != null) { // 启动动作模式
                mModeCallBack.onItemCheckedStateChanged(null, position, id, true); // 通知选中项
                mNotesListView.performHapticFeedback(HapticFeedbackConstants.LONG_PRESS); // 执行触觉反馈
            } else {
                Log.e(TAG, "startActionMode fails"); // 日志输出错误信息
            }
        } else if (mFocusNoteDataItem.getType() == Notes.TYPE_FOLDER) { // 如果为文件夹类型
            mNotesListView.setOnCreateContextMenuListener(mFolderOnCreateContextMenuListener); // 设置上下文菜单监听器
        }
    }
    return false; // 返回false，表示未处理
}
