/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.ui; // 定义包名

import android.content.Context; // 导入Context类
import android.text.format.DateUtils; // 导入DateUtils类
import android.view.View; // 导入View类
import android.widget.CheckBox; // 导入CheckBox类
import android.widget.ImageView; // 导入ImageView类
import android.widget.LinearLayout; // 导入LinearLayout类
import android.widget.TextView; // 导入TextView类

import net.micode.notes.R; // 导入资源类
import net.micode.notes.data.Notes; // 导入Notes数据类
import net.micode.notes.tool.DataUtils; // 导入数据工具类
import net.micode.notes.tool.ResourceParser.NoteItemBgResources; // 导入背景资源解析工具类

public class NotesListItem extends LinearLayout { // 定义NotesListItem类，继承自LinearLayout
    private ImageView mAlert; // 警报图标
    private TextView mTitle; // 笔记标题文本视图
    private TextView mTime; // 笔记时间文本视图
    private TextView mCallName; // 通话记录名称文本视图
    private NoteItemData mItemData; // 当前笔记数据项
    private CheckBox mCheckBox; // 复选框

    public NotesListItem(Context context) { // 构造函数
        super(context); // 调用父类构造函数
        inflate(context, R.layout.note_item, this); // 加载布局文件
        mAlert = (ImageView) findViewById(R.id.iv_alert_icon); // 查找警报图标
        mTitle = (TextView) findViewById(R.id.tv_title); // 查找笔记标题文本视图
        mTime = (TextView) findViewById(R.id.tv_time); // 查找笔记时间文本视图
        mCallName = (TextView) findViewById(R.id.tv_name); // 查找通话记录名称文本视图
        mCheckBox = (CheckBox) findViewById(android.R.id.checkbox); // 查找复选框
    }

    public void bind(Context context, NoteItemData data, boolean choiceMode, boolean checked) { // 绑定视图与数据的方法
        if (choiceMode && data.getType() == Notes.TYPE_NOTE) { // 如果在选择模式且项数据类型为笔记
            mCheckBox.setVisibility(View.VISIBLE); // 显示复选框
            mCheckBox.setChecked(checked); // 设置复选框的选中状态
        } else {
            mCheckBox.setVisibility(View.GONE); // 隐藏复选框
        }

        mItemData = data; // 赋值笔记数据项
        if (data.getId() == Notes.ID_CALL_RECORD_FOLDER) { // 如果是通话记录文件夹
            mCallName.setVisibility(View.GONE); // 隐藏通话记录名称
            mAlert.setVisibility(View.VISIBLE); // 显示警报图标
            mTitle.setTextAppearance(context, R.style.TextAppearancePrimaryItem); // 设置标题文本外观
            mTitle.setText(context.getString(R.string.call_record_folder_name) // 设置标题文本
                    + context.getString(R.string.format_folder_files_count, data.getNotesCount())); // 显示通话记录文件夹名称和数量
            mAlert.setImageResource(R.drawable.call_record); // 设置警报图标为通话记录图标
        } else if (data.getParentId() == Notes.ID_CALL_RECORD_FOLDER) { // 如果父级ID为通话记录文件夹
            mCallName.setVisibility(View.VISIBLE); // 显示通话记录名称
            mCallName.setText(data.getCallName()); // 设置通话记录名称文本
            mTitle.setTextAppearance(context,R.style.TextAppearanceSecondaryItem); // 设置标题文本外观为次要样式
            mTitle.setText(DataUtils.getFormattedSnippet(data.getSnippet())); // 设置标题文本为摘要
            if (data.hasAlert()) { // 如果有警报
                mAlert.setImageResource(R.drawable.clock); // 设置警报图标为时钟
                mAlert.setVisibility(View.VISIBLE); // 显示警报图标
            } else {
                mAlert.setVisibility(View.GONE); // 隐藏警报图标
            }
        } else { // 如果不是通话记录文件夹
            mCallName.setVisibility(View.GONE); // 隐藏通话记录名称
            mTitle.setTextAppearance(context, R.style.TextAppearancePrimaryItem); // 设置标题文本外观为主要样式

            if (data.getType() == Notes.TYPE_FOLDER) { // 如果数据类型为文件夹
                mTitle.setText(data.getSnippet() // 设置标题为文件夹摘要和文件夹中的笔记数量
                        + context.getString(R.string.format_folder_files_count,
                                data.getNotesCount()));
                mAlert.setVisibility(View.GONE); // 隐藏警报图标
            } else { // 如果是笔记类型
                mTitle.setText(DataUtils.getFormattedSnippet(data.getSnippet())); // 设置标题为摘要
                if (data.hasAlert()) { // 如果有警报
                    mAlert.setImageResource(R.drawable.clock); // 设置警报图标为时钟
                    mAlert.setVisibility(View.VISIBLE); // 显示警报图标
                } else {
                    mAlert.setVisibility(View.GONE); // 隐藏警报图标
                }
            }
        }
        mTime.setText(DateUtils.getRelativeTimeSpanString(data.getModifiedDate())); // 设置文本视图为相对时间格式的修改日期

        setBackground(data); // 设置背景
    }

    private void setBackground(NoteItemData data) { // 设置背景的方法
        int id = data.getBgColorId(); // 获取背景颜色ID
        if (data.getType() == Notes.TYPE_NOTE) { // 如果是笔记类型
            if (data.isSingle() || data.isOneFollowingFolder()) { // 如果是唯一笔记或文件夹下唯一笔记
                setBackgroundResource(NoteItemBgResources.getNoteBgSingleRes(id)); // 设置为单个笔记的背景资源
            } else if (data.isLast()) { // 如果为最后一项
                setBackgroundResource(NoteItemBgResources.getNoteBgLastRes(id)); // 设置为最后一项的背景资源
            } else if (data.isFirst() || data.isMultiFollowingFolder()) { // 如果为第一项或多个笔记在文件夹下
                setBackgroundResource(NoteItemBgResources.getNoteBgFirstRes(id)); // 设置为第一项的背景资源
            } else {
                setBackgroundResource(NoteItemBgResources.getNoteBgNormalRes(id)); // 设置为正常背景资源
            }
        } else { // 如果不是笔记类型
            setBackgroundResource(NoteItemBgResources.getFolderBgRes()); // 设置为文件夹背景资源
        }
    }

    public NoteItemData getItemData() { // 获取当前笔记项数据
        return mItemData; // 返回笔记项数据
    }
}