/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.ui;
// 指定当前类的包名，属于net.micode.notes.ui的模块。
import android.accounts.Account;
// 导入Android账户类，用于获取和管理用户账户。
import android.accounts.AccountManager;
// 导入Android账户管理器类，负责账户的操作。
import android.app.ActionBar;
// 导入Android的ActionBar类，用于支持应用显示顶部的导航栏。
import android.app.AlertDialog;
// 导入Android的AlertDialog类，用于创建对话框。
import android.content.BroadcastReceiver;
// 导入Android的BroadcastReceiver类，用于接收广播。
import android.content.ContentValues;
// 导入Android的ContentValues类，用于存储数据库中待插入或更新的值。
import android.content.Context;
// 导入Android的Context类，用于访问应用程序的环境信息。
import android.content.DialogInterface;
// 导入Android的DialogInterface类，用于处理对话框响应的接口。
import android.content.Intent;
// 导入Android的Intent类，用于在组件之间传递信息。
import android.content.IntentFilter;
// 导入Android的IntentFilter类，用于定义接收特定广播的过滤条件。
import android.content.SharedPreferences;
// 导入Android的SharedPreferences类，用于访问应用程序的持久化偏好设置。
import android.os.Bundle;
// 导入Android的Bundle类，用于保存活动状态信息。
import android.preference.Preference;
// 导入Android的Preference类，表示持久化的偏好设置。
import android.preference.Preference.OnPreferenceClickListener;
// 导入Android的OnPreferenceClickListener接口，用于处理偏好设置项的点击事件。
import android.preference.PreferenceActivity;
// 导入Android的PreferenceActivity类，用于显示偏好设置界面。
import android.preference.PreferenceCategory;
// 导入Android的PreferenceCategory类，表示一组相关的偏好设置。
import android.text.TextUtils;
// 导入Android的TextUtils工具类，提供字符串处理方法。
import android.text.format.DateFormat;
// 导入Android的DateFormat类，用于日期格式化。
import android.view.LayoutInflater;
// 导入Android的LayoutInflater类，用于将XML布局文件转换为View对象。
import android.view.Menu;
// 导入Android的Menu类，用于创建菜单。
import android.view.MenuItem;
// 导入Android的MenuItem类，表示菜单中的各个项。
import android.view.View;
// 导入Android的View类，表示视觉组件。
import android.widget.Button;
// 导入Android的Button类，表示按钮组件。
import android.widget.TextView;
// 导入Android的TextView类，表示文本视图组件。
import android.widget.Toast;
// 导入Android的Toast类，用于显示短暂的消息。
import net.micode.notes.R;
// 导入自定义资源文件R，用于获取应用内的资源（如字符串、布局等）。
import net.micode.notes.data.Notes;
// 导入笔记数据相关的类。
import net.micode.notes.data.Notes.NoteColumns;
// 导入笔记数据的列定义。
import net.micode.notes.gtask.remote.GTaskSyncService;
// 导入GTask同步服务，用于管理数据同步过程。
public class NotesPreferenceActivity extends PreferenceActivity {
// 定义一个名为NotesPreferenceActivity的公共类，继承自PreferenceActivity
// 这是用于管理笔记应用的偏好设置界面。
    public static final String PREFERENCE_NAME = "notes_preferences";
// 声明偏好设置文件的名称常量。
    public static final String PREFERENCE_SYNC_ACCOUNT_NAME = "pref_key_account_name";
// 声明用于存储同步账户名称的偏好键的常量。
    public static final String PREFERENCE_LAST_SYNC_TIME = "pref_last_sync_time";
// 声明用于存储最后同步时间的偏好键的常量。
    public static final String PREFERENCE_SET_BG_COLOR_KEY = "pref_key_bg_random_appear";
// 声明用于存储背景颜色随机出现选项的偏好键的常量。
    private static final String PREFERENCE_SYNC_ACCOUNT_KEY = "pref_sync_account_key";
// 声明用于存储同步账户的偏好键的常量。
    private static final String AUTHORITIES_FILTER_KEY = "authorities";
// 声明用于过滤账户权限的键的常量。
    private PreferenceCategory mAccountCategory;
// 声明变量，表示账户类别的偏好设置。
    private GTaskReceiver mReceiver;
// 声明广播接收器变量，用于接收和处理同步相关的广播。
    private Account[] mOriAccounts;
// 声明变量，存储原始账户信息的数组。
    private boolean mHasAddedAccount;
// 声明布尔变量，用于标识用户是否已经添加了新账户。
    @Override
    protected void onCreate(Bundle icicle) {
    // 重写onCreate方法，当活动被创建时调用。
        super.onCreate(icicle);
        // 调用父类的onCreate方法。

        /* using the app icon for navigation */
        getActionBar().setDisplayHomeAsUpEnabled(true);
        // 设置ActionBar支持返回导航（向上导航）。
        addPreferencesFromResource(R.xml.preferences);
        // 从XML资源文件加载应用的偏好设置。
        mAccountCategory = (PreferenceCategory) findPreference(PREFERENCE_SYNC_ACCOUNT_KEY);
        // 找到并初始化用于显示账户的偏好设置类别。
        mReceiver = new GTaskReceiver();
        // 创建GTaskReceiver的实例，处理同步相关的广播。
        IntentFilter filter = new IntentFilter();
        // 创建意图过滤器，用于注册监听特定的广播。
        filter.addAction(GTaskSyncService.GTASK_SERVICE_BROADCAST_NAME);
        // 为过滤器添加要监听的广播动作，即GTask服务的广播。
        registerReceiver(mReceiver, filter);
        // 注册接收器，开始监听同步服务的广播。
        mOriAccounts = null;
        // 初始化原始账户数组为空。
        View header = LayoutInflater.from(this).inflate(R.layout.settings_header, null);
        // 从XML文件加载设置界面的表头视图。
        getListView().addHeaderView(header, null, true);
        // 将表头添加到偏好设置列表视图中。
    }
    @Override
    protected void onResume() {
    super.onResume();
    // 生命周期方法：当活动恢复时调用。

        // need to set sync account automatically if user has added a new account
        if (mHasAddedAccount) {
            // 如果用户添加了新账户，则自动设置同步账户。
            Account[] accounts = getGoogleAccounts();
            // 获取当前Google账户列表。
            if (mOriAccounts != null && accounts.length > mOriAccounts.length) {
                // 如果原始账户不为空且新账户数量大于原账户数量
                for (Account accountNew : accounts) {
                    // 遍历新账户。
                    boolean found = false;
                    // 标识是否找到匹配账户。

                    for (Account accountOld : mOriAccounts) {
                        // 遍历原始账户。
                        if (TextUtils.equals(accountOld.name, accountNew.name)) {
                            // 如果找到了相同的账户名。
                            found = true;
                            break;
                        }
                    }
                    if (!found) {
                        // 如果没有找到相同的账户。
                        setSyncAccount(accountNew.name);
                        // 设置新账户为同步账户。
                        break;
                    }
                }
            }
        }

        refreshUI();
        // 刷新用户界面以显示最新的账户和状态信息。
    }
    @Override
    protected void onDestroy() {
        if (mReceiver != null) {
            unregisterReceiver(mReceiver);
            // 如果接收器不为空，取消注册，以避免内存泄漏。
        }
        super.onDestroy();
        // 调用父类的onDestroy方法。
    }
    private void loadAccountPreference() {
    // 加载账户偏好设置的方法。
        mAccountCategory.removeAll();
        // 移除账户类别中的所有偏好设置项。

        Preference accountPref = new Preference(this);
        // 创建新的偏好设置项，用于显示账户信息。
        final String defaultAccount = getSyncAccountName(this);
        // 获取当前同步账户的名称。
        
        accountPref.setTitle(getString(R.string.preferences_account_title));
        // 设置偏好设置项的标题。
        accountPref.setSummary(getString(R.string.preferences_account_summary));
        // 设置偏好设置项的摘要描述。
        accountPref.setOnPreferenceClickListener(new OnPreferenceClickListener() {
            public boolean onPreferenceClick(Preference preference) {
                if (!GTaskSyncService.isSyncing()) {
                    // 如果当前没有进行同步操作。
                    if (TextUtils.isEmpty(defaultAccount)) {
                        // 如果当前没有默认账户。
                        showSelectAccountAlertDialog();
                        // 显示选择账户的对话框。
                    } else {
                        // 如果已有默认账户。
                        showChangeAccountConfirmAlertDialog();
                        // 显示更改账户的确认对话框。
                    }
                } else {
                    Toast.makeText(NotesPreferenceActivity.this,
                            R.string.preferences_toast_cannot_change_account, Toast.LENGTH_SHORT)
                            .show();
                    // 提示用户正在同步时无法更改账户。
                }
                return true; // 表示事件已处理。
            }
        });

        mAccountCategory.addPreference(accountPref);
        // 将创建的账户偏好设置项添加到账户类别中。

    }
    private void loadSyncButton() {
        // 加载同步按钮的方法。
        Button syncButton = (Button) findViewById(R.id.preference_sync_button);
        // 获取用于同步的按钮实例。
        TextView lastSyncTimeView = (TextView) findViewById(R.id.prefenerece_sync_status_textview);
        // 获取用于显示最后同步时间的文本视图。

        // set button state
        if (GTaskSyncService.isSyncing()) {
            // 如果当前正在进行同步。
            syncButton.setText(getString(R.string.preferences_button_sync_cancel));
            // 设置按钮文本为“取消同步”。
            syncButton.setOnClickListener(new View.OnClickListener() {
                public void onClick(View v) {
                    GTaskSyncService.cancelSync(NotesPreferenceActivity.this);
                    // 点击按钮时取消正在进行的同步。
                }
            });
        } else {
            syncButton.setText(getString(R.string.preferences_button_sync_immediately));
            // 设置按钮文本为“立即同步”。
            syncButton.setOnClickListener(new View.OnClickListener() {
                public void onClick(View v) {
                    GTaskSyncService.startSync(NotesPreferenceActivity.this);
                    // 点击按钮时开始同步。
                }
            });
        }
        
        syncButton.setEnabled(!TextUtils.isEmpty(getSyncAccountName(this)));
        // 如果同步账户不为空，则启用按钮。

        // set last sync time
        if (GTaskSyncService.isSyncing()) {
            // 如果正在同步。
            lastSyncTimeView.setText(GTaskSyncService.getProgressString());
            // 设置文本视图的文本为当前同步的进度信息。
            lastSyncTimeView.setVisibility(View.VISIBLE);
        } else {
            long lastSyncTime = getLastSyncTime(this);
            // 获取最后同步时间。
            if (lastSyncTime != 0) {
                lastSyncTimeView.setText(getString(R.string.preferences_last_sync_time,
                        DateFormat.format(getString(R.string.preferences_last_sync_time_format),
                                lastSyncTime)));
                // 设置文本视图的文本为最后同步的时间格式化字符串。
                lastSyncTimeView.setVisibility(View.VISIBLE);
            } else {
                lastSyncTimeView.setVisibility(View.GONE);
                // 如果没有最后同步时间，则隐藏文本视图。
            }
        }
    }
    private void refreshUI() {
        // 刷新用户界面的方法。
        loadAccountPreference();
        // 重新加载账户偏好设置项。
        loadSyncButton();
        // 重新加载同步按钮。
    }
    private void showSelectAccountAlertDialog() {
        // 显示选择账户的对话框的方法。
        AlertDialog.Builder dialogBuilder = new AlertDialog.Builder(this);
        // 创建对话框构建器。

        View titleView = LayoutInflater.from(this).inflate(R.layout.account_dialog_title, null);
        // 从XML布局文件加载自定义对话框标题视图。
        TextView titleTextView = (TextView) titleView.findViewById(R.id.account_dialog_title);
        titleTextView.setText(getString(R.string.preferences_dialog_select_account_title));
        // 设置对话框标题的文本。
        TextView subtitleTextView = (TextView) titleView.findViewById(R.id.account_dialog_subtitle);
        subtitleTextView.setText(getString(R.string.preferences_dialog_select_account_tips));
        // 设置对话框副标题的文本。

        dialogBuilder.setCustomTitle(titleView);
        // 设置对话框的自定义标题。
        dialogBuilder.setPositiveButton(null, null);
        // 设置正面按钮为空，因后续不需要使用。

        Account[] accounts = getGoogleAccounts();
        // 获取所有Google账户。
        String defAccount = getSyncAccountName(this);
        // 获取当前同步账户的名称。

        mOriAccounts = accounts;
        // 存储原始账户。
        mHasAddedAccount = false;
        // 重置添加账户的标记。

        if (accounts.length > 0) {
            // 如果有可用账户。
            CharSequence[] items = new CharSequence[accounts.length];
            // 创建可选账户名称的字符序列数组。
            final CharSequence[] itemMapping = items;
            int checkedItem = -1;
            // 记录当前选中账户的索引。
            int index = 0;
            for (Account account : accounts) {
                if (TextUtils.equals(account.name, defAccount)) {
                    checkedItem = index;
                    // 如果当前账户是同步账户，则记录索引。
                }
                items[index++] = account.name;
                // 将账户名称添加到字符序列数组。
            }
            dialogBuilder.setSingleChoiceItems(items, checkedItem,
                    new DialogInterface.OnClickListener() {
                        public void onClick(DialogInterface dialog, int which) {
                            setSyncAccount(itemMapping[which].toString());
                            // 设置选中的账户作为同步账户。
                            dialog.dismiss();
                            // 关闭对话框。
                            refreshUI();
                            // 刷新用户界面以更新账户信息。
                        }
                    });
        }

        View addAccountView = LayoutInflater.from(this).inflate(R.layout.add_account_text, null);
        // 加载添加账户的视图。
        dialogBuilder.setView(addAccountView);
        // 将添加账户视图设置为对话框内容。

        final AlertDialog dialog = dialogBuilder.show();
        // 显示对话框。

        addAccountView.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                mHasAddedAccount = true;
                // 设置标记为账户已被添加。
                Intent intent = new Intent("android.settings.ADD_ACCOUNT_SETTINGS");
                // 创建意图以跳转到添加账户设置界面。
                intent.putExtra(AUTHORITIES_FILTER_KEY, new String[] {
                    "gmail-ls"
                });
                // 设置过滤器以限制账户类型为 GMail。
                startActivityForResult(intent, -1);
                // 启动账户添加设置界面。
                dialog.dismiss();
                // 关闭对话框。
            }
        });
    }
    private void showChangeAccountConfirmAlertDialog() {
        // 显示更改账户确认对话框的方法。
        AlertDialog.Builder dialogBuilder = new AlertDialog.Builder(this);
        // 创建对话框构建器。

        View titleView = LayoutInflater.from(this).inflate(R.layout.account_dialog_title, null);
        // 从布局中加载自定义标题视图。
        TextView titleTextView = (TextView) titleView.findViewById(R.id.account_dialog_title);
        titleTextView.setText(getString(R.string.preferences_dialog_change_account_title,
                getSyncAccountName(this)));
        // 设置更改账户的标题。
        TextView subtitleTextView = (TextView) titleView.findViewById(R.id.account_dialog_subtitle);
        subtitleTextView.setText(getString(R.string.preferences_dialog_change_account_warn_msg));
        // 设置警告消息的副标题。

        dialogBuilder.setCustomTitle(titleView);
        // 设置对话框的自定义标题。

        CharSequence[] menuItemArray = new CharSequence[] {
                getString(R.string.preferences_menu_change_account),
                getString(R.string.preferences_menu_remove_account),
                getString(R.string.preferences_menu_cancel)
        };
        // 定义菜单项数组，包含更改账户、移除账户和取消选项。

        dialogBuilder.setItems(menuItemArray, new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int which) {
                if (which == 0) {
                    showSelectAccountAlertDialog();
                    // 如果选择更改账户，显示选择账户对话框。
                } else if (which == 1) {
                    removeSyncAccount();
                    // 如果选择移除账户，调用移除账户方法。
                    refreshUI();
                    // 刷新UI以更新信息。
                }
            }
        });
        dialogBuilder.show();
        // 显示对话框。
    }
    private Account[] getGoogleAccounts() {
        AccountManager accountManager = AccountManager.get(this);
        // 获取账户管理器实例。
        return accountManager.getAccountsByType("com.google");
        // 返回Google账户类型的账户数组。
    }
    private void setSyncAccount(String account) {
        if (!getSyncAccountName(this).equals(account)) {
            // 如果未选择到相同的账户。
            SharedPreferences settings = getSharedPreferences(PREFERENCE_NAME, Context.MODE_PRIVATE);
            // 获取偏好设置实例。
            SharedPreferences.Editor editor = settings.edit();
            // 获取编辑器，以便存储偏好值。
            if (account != null) {
                editor.putString(PREFERENCE_SYNC_ACCOUNT_NAME, account);
                // 如果账户不为空，存储账户名。
            } else {
                editor.putString(PREFERENCE_SYNC_ACCOUNT_NAME, "");
                // 如果账户为空，存储空字符串。
            }
            editor.commit();
            // 提交编辑器的更改。

            // clean up last sync time
            setLastSyncTime(this, 0);
            // 清除最后同步时间。

            // clean up local gtask related info
            new Thread(new Runnable() {
                public void run() {
                    ContentValues values = new ContentValues();
                    values.put(NoteColumns.GTASK_ID, "");
                    values.put(NoteColumns.SYNC_ID, 0);
                    // 清空GTASK相关数据。
                    getContentResolver().update(Notes.CONTENT_NOTE_URI, values, null, null);
                    // 更新笔记中的相关内容。
                }
            }).start();

            Toast.makeText(NotesPreferenceActivity.this,
                    getString(R.string.preferences_toast_success_set_accout, account),
                    Toast.LENGTH_SHORT).show();
            // 显示成功设置账户的提示。
        }
    }
    private void removeSyncAccount() {
        SharedPreferences settings = getSharedPreferences(PREFERENCE_NAME, Context.MODE_PRIVATE);
        // 获取偏好设置实例。
        SharedPreferences.Editor editor = settings.edit();
        // 获取编辑器。
        if (settings.contains(PREFERENCE_SYNC_ACCOUNT_NAME)) {
            editor.remove(PREFERENCE_SYNC_ACCOUNT_NAME);
            // 如果偏好设置中包含同步账户名，移除它。
        }
        if (settings.contains(PREFERENCE_LAST_SYNC_TIME)) {
            editor.remove(PREFERENCE_LAST_SYNC_TIME);
            // 如果偏好设置中包含最后同步时间，移除它。
        }
        editor.commit();
        // 提交编辑器的更改。

        // clean up local gtask related info
        new Thread(new Runnable() {
            public void run() {
                ContentValues values = new ContentValues();
                values.put(NoteColumns.GTASK_ID, "");
                values.put(NoteColumns.SYNC_ID, 0);
                // 清空GTASK相关数据。
                getContentResolver().update(Notes.CONTENT_NOTE_URI, values, null, null);
                // 更新笔记中相关内容。
            }
        }).start();
    }
    public static String getSyncAccountName(Context context) {
        SharedPreferences settings = context.getSharedPreferences(PREFERENCE_NAME,
                Context.MODE_PRIVATE);
        // 获取偏好设置实例。
        return settings.getString(PREFERENCE_SYNC_ACCOUNT_NAME, "");
        // 返回当前同步账户名称，默认为空字符串。
    }
    public static void setLastSyncTime(Context context, long time) {
        SharedPreferences settings = context.getSharedPreferences(PREFERENCE_NAME,
                Context.MODE_PRIVATE);
        // 获取偏好设置实例。
        SharedPreferences.Editor editor = settings.edit();
        editor.putLong(PREFERENCE_LAST_SYNC_TIME, time);
        // 存储最后同步时间。
        editor.commit();
        // 提交编辑器的更改。
    }
    public static long getLastSyncTime(Context context) {
        SharedPreferences settings = context.getSharedPreferences(PREFERENCE_NAME,
                Context.MODE_PRIVATE);
        // 获取偏好设置实例。
        return settings.getLong(PREFERENCE_LAST_SYNC_TIME, 0);
        // 返回最后同步时间，默认为0。
    }
    private class GTaskReceiver extends BroadcastReceiver {
        // 定义GTaskReceiver类，用于接收同步服务的广播。

        @Override
        public void onReceive(Context context, Intent intent) {
            // 当接收到广播时调用。
            refreshUI();
            // 刷新UI以显示最新信息。
            if (intent.getBooleanExtra(GTaskSyncService.GTASK_SERVICE_BROADCAST_IS_SYNCING, false)) {
                TextView syncStatus = (TextView) findViewById(R.id.prefenerece_sync_status_textview);
                // 获取显示同步状态的文本视图。
                syncStatus.setText(intent
                        .getStringExtra(GTaskSyncService.GTASK_SERVICE_BROADCAST_PROGRESS_MSG));
                // 设置文本视图内容为同步进度信息。
            }
        }
    }
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case android.R.id.home:
                Intent intent = new Intent(this, NotesListActivity.class);
                intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
                // 创建意图，跳转到笔记列表活动，并清除栈中的其它活动。
                startActivity(intent);
                // 启动活动。
                return true;
            default:
                return false;
            // 如果不是默认情形，返回false。
        }
    }
}